/**
 * @license
 * Copyright Akveo. All Rights Reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 */
import { BrowserModule } from '@angular/platform-browser';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations';
import { APP_INITIALIZER, LOCALE_ID, NgModule, NgZone, PLATFORM_ID } from '@angular/core';
import { HttpClientModule, HTTP_INTERCEPTORS } from '@angular/common/http';
import { AppComponent } from './app.component';
import { AppRoutingModule } from './app-routing.module';
import {
  NbAlertModule,
  NbDatepickerModule,
  NbDialogModule,
  NbMenuModule,
  NbSidebarModule,
  NbTimepickerModule,
  NbToastrModule,
  NbWindowModule,
} from '@nebular/theme';

import { registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import { environment } from 'environments/environment';

import { SharedModule } from './shared/shared.module';
import { KeycloakAngularModule, KeycloakService } from 'keycloak-angular';
import { CoreModule } from './core/core.module';
import { MathjaxModule } from '@sls/mathjax-angular';
import { RouteReuseStrategy } from '@angular/router';
import { CustomReuseStrategy } from './core/custom-reuse-strategy';
import { HttpCacheInterceptorModule, HTTP_CACHE_CONFIG } from '@ngneat/cashew';

registerLocaleData(localeDe);

function initializeKeycloak(keycloak: KeycloakService) {
  return () =>
    keycloak.init({
      config: {
        url: environment.keycloakUrl,
        realm: 'SLS',
        clientId: 'subato'
      },
      initOptions: {
        onLoad: 'check-sso',
        // disable silent check in production because the SBI proxy is blocking iframes (X-Frame-Options: DENY)
        silentCheckSsoRedirectUri: environment.production ? undefined : window.location.origin + '/assets/silent-check-sso.html'
      },
      shouldAddToken: (request) => {
        return request.url.startsWith(environment.backendUrl);
      }
    });
}

function cashewConfig() {
  return {
    strategy: 'implicit',
    ttl: 0
  }
}

@NgModule({
  declarations: [AppComponent],
  imports: [
    BrowserModule,
    BrowserAnimationsModule,
    HttpClientModule,
    KeycloakAngularModule,
    AppRoutingModule,
    MathjaxModule.forRoot(),
    NbSidebarModule.forRoot(),
    NbMenuModule.forRoot(),
    NbDatepickerModule.forRoot(),
    NbTimepickerModule.forRoot(),
    NbDialogModule.forRoot(),
    NbWindowModule.forRoot(),
    NbToastrModule.forRoot(),
    CoreModule.forRoot(),
    SharedModule,
    NbAlertModule,    
    NbToastrModule.forRoot(),
    HttpCacheInterceptorModule.forRoot()
  ],
  bootstrap: [AppComponent],
  providers: [
    { provide: LOCALE_ID, useValue: 'de-DE' },
    {
      provide: APP_INITIALIZER,
      useFactory: initializeKeycloak,
      multi: true,
      deps: [KeycloakService]
    },
    { provide: RouteReuseStrategy, useClass: CustomReuseStrategy},
    { provide: HTTP_CACHE_CONFIG, useValue: cashewConfig() }
  ]
})
export class AppModule {
}
