import { Component, OnInit, ViewChild } from '@angular/core';
import { FormBuilder } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { NbDialogService } from '@nebular/theme';
import { SubatoPrincipal } from 'app/core/models/principal/subato-principal';
import { User } from 'app/core/models/user';
import { AppStateService } from 'app/core/services/app.state.service';
import { AuthService } from 'app/core/services/auth.service';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { ErrorContainerConfig } from 'app/shared/components/error-container/error-container.component';
import { BackendError } from 'app/shared/models/backend-error';
import { NotificationService } from 'app/shared/services/notification.service';
import { combineLatest } from 'rxjs';
import { takeUntil, flatMap } from 'rxjs/operators';
import { CourseFileListComponent } from '../../components/course-file-list/course-file-list.component';
import { Course } from '../../models/course';
import { CourseFile } from '../../models/course-file';
import { CourseService } from '../../services/course.service';

@Component({
  selector: 'ngx-course-files',
  templateUrl: './course-files.component.html',
  styleUrls: ['./course-files.component.scss']
})
export class CourseFilesComponent extends BaseComponent implements OnInit {
  @ViewChild(CourseFileListComponent, { static: false }) courseFilesComponent!: CourseFileListComponent;

  loading = true;
  course: Course;
  principal: SubatoPrincipal;

  error: BackendError;
  errorConfig: ErrorContainerConfig = {
    notFound: {
      message: 'Diesen Kurs konnten wir leider nicht finden.'
    }
  };

  constructor(
    private courseService: CourseService,
    private appStateService: AppStateService,
    private router: Router,
    private authService: AuthService,
    private dialogService: NbDialogService,
    private notificationService: NotificationService,
    private route: ActivatedRoute) {
    super();
  }

  ngOnInit(): void {
    combineLatest(this.route.paramMap, this.authService.principal$)
      .pipe(
        takeUntil(this.unsubscribe),
        flatMap(([params, principal]) => {
          this.principal = principal;
          let courseId = Number(params.get("course_id"));
          return this.courseService.getById(courseId);
        })
      )
      .subscribe(course => {
        this.course = course;
        this.appStateService.setCourse(course);

        this.loading = false;
      }, error => {
        if (BackendError.isBackendError(error)) {
          this.error = error;
          this.loading = false;
        } else {
          throw error;
        }
      });
  }

  onFileUploaded(file: CourseFile) {
    this.courseFilesComponent.refresh();
  }
}
