import { Component, OnInit } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { AuthService } from 'app/core/services/auth.service';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { NotificationService } from 'app/shared/services/notification.service';
import { combineLatest, forkJoin, of } from 'rxjs';
import { takeUntil, flatMap, filter, catchError } from 'rxjs/operators';
import { AppStateService } from 'app/core/services/app.state.service';
import { CourseService } from '../../services/course.service';
import { Course } from '../../models/course';
import { BackendError } from 'app/shared/models/backend-error';
import { ErrorContainerConfig } from 'app/shared/components/error-container/error-container.component';
import { DistinctHelper } from 'app/shared/helper/distinct.helper';
import { SubatoPrincipal } from 'app/core/models/principal/subato-principal';

@Component({
  selector: 'ngx-course',
  templateUrl: './course.component.html',
  styleUrls: ['./course.component.scss']
})
export class CourseComponent extends BaseComponent implements OnInit {
  course: Course;
  principal: SubatoPrincipal;
  loading: boolean;
  
  error: BackendError;
  errorConfig: ErrorContainerConfig = {
    notFound: {
      message: 'Diesen Kurs konnten wir leider nicht finden.'
    }
  };

  constructor(
    private courseService: CourseService,
    private router: Router,
    private appStateService: AppStateService,
    private authService: AuthService,
    private notificationService: NotificationService,
    private route: ActivatedRoute) {
    super();
  }

  ngOnInit(): void {
    this.loading = true;
    
    combineLatest(this.route.paramMap, this.authService.principal$)
      .pipe(
        takeUntil(this.unsubscribe),
        flatMap(([params, principal]) => {
          this.principal = principal;
          let courseId = Number(params.get("course_id"));

          return this.courseService.getById(courseId);
        }),
      )
      .subscribe(course => {
        this.course = course;
        this.appStateService.setCourse(course);
        
        this.loading = false;
      }, (error) => {
        if (BackendError.isBackendError(error)) {
          this.error = error;
          this.loading = false;
        } else {
          throw error;
        }
      });
  }

  get programs() {
    return DistinctHelper.distinct(this.course.modules.map(m => m.program), (program) => program.id);
  }
}
