import { NgModule } from '@angular/core';
import { Routes, RouterModule, ActivatedRouteSnapshot } from '@angular/router';
import { User } from 'app/core/models/user';
import { Exercise } from './models/exercise';
import { Submission } from './models/submission';
import { TaskInstance } from './models/task-instance';
import { EditExerciseComponent } from './pages/edit-exercise/edit-exercise.component';
import { ExerciseComponent } from './pages/exercise/exercise.component';
import { ExercisesComponent } from './pages/exercises/exercises.component';
import { NewExerciseComponent } from './pages/new-exercise/new-exercise.component';
import { SubmissionComponent } from './pages/submission/submission.component';
import { SubmissionsComponent } from './pages/submissions/submissions.component';
import { SubmitSolutionComponent } from './pages/submit-solution/submit-solution.component';
import { TaskInstanceSubmissionsComponent } from './pages/task-instance-submissions/task-instance-submissions.component';

export const routes: Routes = [
  {
    path: 'course/:course_id/exercises',
    component: ExercisesComponent,
    data: {
      breadcrumb: {
        alias: 'exercises',
        label: 'Übungen' 
      },
      courseMenuEnabled: true
    }
  },
  {
    path: 'course/:course_id/exercises/new',
    component: NewExerciseComponent,
    data: {
      authRequired: true,
      breadcrumb: {
        alias: 'newExercise',
        label: 'Erstellen' 
      },
      courseMenuEnabled: true
    }
  },
  {
    path: 'exercises/:exercise_id',
    component: ExerciseComponent,
    data: {
      breadcrumb: {
        alias: 'exercise',
        label: (exercise: Exercise) => `${exercise.name}` 
      },
      courseMenuEnabled: true
    }
  },
  {
    path: 'exercises/:exercise_id/edit',
    component: EditExerciseComponent,
    data: {
      authRequired: true,
      breadcrumb: {
        alias: 'editExercise',
        label: `Bearbeiten`
      },
      courseMenuEnabled: true
    }
  },
  {
    path: 'exercises/:exercise_id/submissions',
    component: SubmissionsComponent,
    data: {
      authRequired: true,
      breadcrumb: {
        alias: 'submissions',
        label: `Bewertung`
      },
      courseMenuEnabled: true
    }
  },
  
  {
    path: 'submissions/:submission_id',
    component: SubmissionComponent,
    data: {
      authRequired: true,
      breadcrumb: {
        alias: 'submission',
        label: (submission: Submission) => `Abgabe von ${submission.submittedBy.fixedName}` 
      },
      courseMenuEnabled: true
    }
  },
  {
    path: 'exercises/:exercise_id/tasks/:task_id',
    component: SubmitSolutionComponent,
    data: {
      breadcrumb: {
        alias: 'submitSolution',
        label: (ti: TaskInstance) => `${ti.task.name}` 
      },
      courseMenuEnabled: true
    }
  },
  {
    path: 'exercises/:exercise_id/tasks/:task_id/submissions',
    component: TaskInstanceSubmissionsComponent,
    data: {
      authRequired: true,
      breadcrumb: {
        alias: 'taskInstanceSubmissions',
        label: `Bewertung`
      },
      courseMenuEnabled: true
    }
  },
];


@NgModule({
  exports: [RouterModule]
})
export class ExercisesRoutingModule { }
