import { Component, OnInit } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { AuthService } from 'app/core/services/auth.service';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { combineLatest } from 'rxjs';
import { flatMap, takeUntil } from 'rxjs/operators';
import { Course } from 'app/features/courses/models/course';
import { CourseService } from 'app/features/courses/services/course.service';
import { User } from 'app/core/models/user';
import { BackendError } from 'app/shared/models/backend-error';
import { ErrorContainerConfig } from 'app/shared/components/error-container/error-container.component';
import { AppStateService } from 'app/core/services/app.state.service';
import { Breadcrumb } from 'app/shared/components/breadcrumbs/breadcrumbs.component';
import { SubatoPrincipal } from 'app/core/models/principal/subato-principal';

@Component({
  selector: 'ngx-exercises',
  templateUrl: './exercises.component.html',
  styleUrls: ['./exercises.component.scss']
})
export class ExercisesComponent extends BaseComponent implements OnInit {
  course: Course;
  
  loading: boolean;
  principal: SubatoPrincipal;
  error: BackendError;
  errorConfig: ErrorContainerConfig = {
    notFound: {
      message: 'Diesen Kurs konnten wir leider nicht finden.'
    }
  };

  breadcrumbs: Breadcrumb[] = [];
  
  constructor(
    private courseService: CourseService,
    private appStateService: AppStateService,
    private authService: AuthService,
    private route: ActivatedRoute) {
    super();
  }

  buildBreadcrumbs(course: Course) {
    this.breadcrumbs = [
      {
        id: 'exercises',
        params: {
          course_id: course.id
        },
      },
    ];
  }

  ngOnInit(): void {
    this.loading = true;
    combineLatest(this.route.paramMap, this.authService.principal$)
      .pipe(
        takeUntil(this.unsubscribe),
        flatMap(([params, principal]) => {
          this.principal = principal;
          let courseId = Number(params.get("course_id"));
          
          return this.courseService.getById(courseId);
        })
      ).subscribe(course => {
        this.course = course;
        this.appStateService.setCourse(course);
        this.buildBreadcrumbs(course);
        
        this.loading = false;
      }, error => {
        if (BackendError.isBackendError(error)) {
          this.error = error;
          this.loading = false;
        } else {
          throw error;
        }
      });
  }
}
