import { Component, OnInit } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { ErrorContainerConfig } from 'app/shared/components/error-container/error-container.component';
import { BackendError } from 'app/shared/models/backend-error';
import { NotificationService } from 'app/shared/services/notification.service';
import { Observable, of } from 'rxjs';
import { takeUntil, flatMap } from 'rxjs/operators';
import { Term } from '../../models/term';
import { TermService } from '../../services/term.service';

@Component({
  selector: 'ngx-edit-term',
  templateUrl: './edit-term.component.html',
  styleUrls: ['./edit-term.component.scss']
})
export class EditTermComponent extends BaseComponent implements OnInit {
  loading: boolean = true;
  term: Term;

  error: BackendError;
  errorConfig: ErrorContainerConfig = {
    notFound: {
      message: 'Dieses Semester konnten wir leider nicht finden.'
    }
  };

  constructor(
    private termService: TermService,
    private router: Router,
    private route: ActivatedRoute,
    private notificationService: NotificationService) {
    super();
  }

  ngOnInit(): void {
    this.route.paramMap
      .pipe(
        takeUntil(this.unsubscribe),
        flatMap(params => {
          let termId = params.has("term_id") ? Number(params.get("term_id")) : null;

          let loadPool: Observable<Term> = of(null);
          if (termId != null) {
            loadPool = this.termService.getById(termId);
          }

          return loadPool;
        })
      ).subscribe(term => {
        this.term = term;
        this.loading = false;
      }, error => {
        if (BackendError.isBackendError(error)) {
          this.error = error;
          this.loading = false;
        } else {
          throw error;
        }
      });
  }
}
