import { Injectable, ViewContainerRef } from "@angular/core";
import { Solution } from "app/features/solutions/models/solution";
import { BaseComponent } from "app/shared/components/base/base.component";
import { FileTree } from "app/shared/models/file-tree";
import { Observable } from "rxjs";
import { map } from "rxjs/operators";
import { DefaultViewerFactory } from "./default-viewer/default-viewer.factory";
import { NothingViewerFactory } from "./nothing-viewer/nothing-viewer.factory";
import { PdfViewerFactory } from "./pdf-viewer/pdf-viewer.factory";
import { SourceCodeViewerFactory } from "./source-code-viewer/source-code-viewer.factory";
import { Viewer } from "./viewer";

@Injectable({
    providedIn: 'root'
})
export class ViewerFactory extends BaseComponent {
    constructor(
        private sourceCodeViewerFactory: SourceCodeViewerFactory,
        private pdfViewerFactory: PdfViewerFactory,
        private nothingViewerFactory: NothingViewerFactory,
        private defaultViewerFactory: DefaultViewerFactory,
    ) {
        super();
    }

    private getViewer(tree: FileTree): Viewer {
        if (tree == null || tree.directory) {
            return Viewer.none;
        }

        if (tree.src) {
            return Viewer.sourceCode;
        }

        if (!tree.src && tree.isPdf) {
            return Viewer.pdf;
        }

        return Viewer.default;
    }

    createViewer(solution: Solution, tree: FileTree, containerRef: ViewContainerRef): Observable<Viewer> {

        let viewer = this.getViewer(tree);
        let create$: Observable<void>;

        if (viewer == Viewer.none) {
            create$ = this.nothingViewerFactory.createViewer(containerRef);
        }

        else if (viewer == Viewer.pdf) {
            create$ = this.pdfViewerFactory.createViewer(solution, tree, containerRef);
        }

        else if (viewer == Viewer.sourceCode) {
            create$ = this.sourceCodeViewerFactory.createViewer(solution, tree, containerRef);
        }

        else if (viewer == Viewer.default) {
            create$ = this.defaultViewerFactory.createViewer(solution, tree, containerRef);
        }

        else {
            throw `no factory for viewer ${viewer}`;
        }

        return create$.pipe(
            map(() => viewer)
        );
    }
}