import { HttpClient } from "@angular/common/http";
import { Injectable } from "@angular/core";
import { BaseService } from "app/shared/services/base.service";
import { environment } from "environments/environment";
import { forkJoin, Observable, of } from "rxjs";
import { catchError, flatMap, map } from "rxjs/operators";
import { CreateSolutionCommentDto } from "../dto/create-solution-comment.dto";
import { SolutionComment } from "../models/solution-comment";
import { BackendError } from "app/shared/models/backend-error";

@Injectable({
    providedIn: 'root'
})
export class SolutionCommentService extends BaseService {
    constructor(private http: HttpClient) {
        super();
    }

    create(dto: CreateSolutionCommentDto): Observable<SolutionComment> {
        return this.http.post<SolutionComment>(`${environment.backendUrl}/solution_comment`, dto).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(comment => new SolutionComment(comment))
        );
    }

    delete(id: number): Observable<void> {
        return this.http.delete<void>(`${environment.backendUrl}/solution_comment/${id}`)
            .pipe(
                catchError(e => {
                    throw BackendError.parse(e);
                }),
            );
    }
}