import { Options } from '@angular-slider/ngx-slider';
import { Component, Input, OnInit, Output } from '@angular/core';
import { UntypedFormBuilder, UntypedFormControl, UntypedFormGroup } from '@angular/forms';
import { SubatoPrincipal } from 'app/core/models/principal/subato-principal';
import { User } from 'app/core/models/user';
import { AuthService } from 'app/core/services/auth.service';
import { CreateTaskFeedbackDto } from 'app/features/tasks/dto/create-task-feedback.dto';
import { TaskFeedback } from 'app/features/tasks/models/task-feedback';
import { TaskService } from 'app/features/tasks/services/task.service';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { NotificationService } from 'app/shared/services/notification.service';
import { of } from 'rxjs';
import { takeUntil, flatMap, map, catchError } from 'rxjs/operators';

@Component({
  selector: 'ngx-feedback',
  templateUrl: './feedback.component.html',
  styleUrls: ['./feedback.component.scss']
})
export class FeedbackComponent extends BaseComponent implements OnInit {
  @Input()
  onSubmit: any;

  @Input()
  self: any;

  @Input()
  taskId: number;

  loading: boolean;
  principal: SubatoPrincipal;

  feedbackForm: UntypedFormGroup;
  comprehensibilityOptions: Options = {
    floor: 1,
    ceil: 5,
    showSelectionBar: true,
    showTicks: true
  };

  complexityOptions: Options = {
    floor: 1,
    ceil: 5,
    showSelectionBar: true,
    showTicks: true
  };
  
  constructor(
    private fb: UntypedFormBuilder, 
    private taskService: TaskService,
    private authService: AuthService,
    private notificationService: NotificationService
  ) { 
    super();
  }

  ngOnInit(): void {
    this.loading = true;
    this.authService.principal$
    .pipe(
      takeUntil(this.unsubscribe),
      flatMap(principal => {
        this.principal = principal;
        return this.taskService.getFeedbackByUser(this.taskId, principal.id).pipe(catchError(e => of(null)));
      })
    )
    .subscribe(feedback => {
      this.createForm(feedback);
      this.loading = false;
    });
  }

  createForm(feedback: TaskFeedback | null) {
    this.feedbackForm = this.fb.group({
      comprehensibility: new UntypedFormControl(feedback?.comprehensibility ?? 3),
      complexity: new UntypedFormControl(feedback?.complexity ?? 3),
      comment: [feedback?.comment ?? '']
    });
  }

  submit() {
    let dto: CreateTaskFeedbackDto = {
      comprehensibility: this.feedbackForm.get('comprehensibility').value,
      complexity: this.feedbackForm.get('complexity').value,
      comment: this.feedbackForm.get('comment').value,
    };

    this.taskService.createFeedback(this.taskId, dto)
    .pipe(
      takeUntil(this.unsubscribe)
    ).subscribe(feedback => {
      this.onSubmit.call(this.self);
      this.notificationService.showSuccess("Vielen Dank für deine Rückmeldung!");
    });
  }
}
