import { Component, OnInit } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { BackendError } from 'app/shared/models/backend-error';
import { takeUntil } from 'rxjs/operators';
import { TaskPool } from '../../models/task-pool';
import { TaskPoolService } from '../../services/task-pool.service';

@Component({
  selector: 'ngx-task-pool-list',
  templateUrl: './task-pool-list.component.html',
  styleUrls: ['./task-pool-list.component.scss']
})
export class TaskPoolListComponent extends BaseComponent implements OnInit {
  taskPools: TaskPool[] = [];

  loading: boolean = true;
  error: BackendError;

  constructor(
    private taskPoolService: TaskPoolService,
    private router: Router,
    private route: ActivatedRoute) {
    super();
  }

  ngOnInit(): void {
    this.taskPoolService.getAll()
    .pipe(
      takeUntil(this.unsubscribe)
    )
    .subscribe(taskPools => {
      this.taskPools = taskPools;
      this.loading = false;
    }, error => {
      if (BackendError.isBackendError(error)) {
        this.error = error;
        this.loading = false;
      } else {
        throw error;
      }
    });
  }
}
