import { ProgramModule } from "app/features/modules/models/program-module";
import { Language } from "./language";
import { SubmissionMode } from "./submission-mode";
import { TaskFeedbackStats } from "./task-feedback-stats";
import { FileRef } from "./file-ref";
import { TaskPool } from "./task-pool";
import { Evaluator } from "./evaluator";

export class Task {
    id: number;
    idName: string;
    name: string;
    description: string;
    attempts: number | null;
    defaultSubmissionMode: SubmissionMode;
    supportsFileSubmissionMode: boolean;
    supportsTextSubmissionMode: boolean;
    supportedByEclipsePlugin: boolean;
    language: Language;
    pool: TaskPool;
    feedbackStats: TaskFeedbackStats;
    acceptanceTestAvailable: boolean;
    evaluator: Evaluator;
    repositoryUrl: string;
    allowedExtensions: string[];

    constructor(task: any) {
        this.id = task.id;
        this.idName = task.idName;
        this.name = task.name;
        this.description = task.description;
        this.attempts = task.attempts;
        this.defaultSubmissionMode = task.defaultSubmissionMode;
        this.supportsFileSubmissionMode = task.supportsFileSubmissionMode;
        this.supportsTextSubmissionMode = task.supportsTextSubmissionMode;
        this.feedbackStats = task.feedbackStats;
        this.evaluator = task.evaluator ? new Evaluator(task.evaluator) : null;
        this.pool = task.pool ? new TaskPool(task.pool) : null;
        this.language = task.language ? new Language(task.language) : null;
        this.acceptanceTestAvailable = task.acceptanceTestAvailable;
        this.supportedByEclipsePlugin = task.supportedByEclipsePlugin;
        this.repositoryUrl = task.repositoryUrl;
        this.allowedExtensions = task.allowedExtensions;
    }
}