package de.hsrm.sls.subato.auth.policy;

import de.hsrm.sls.subato.auth.principal.SubatoPrincipal;
import de.hsrm.sls.subato.courses.course.Course_;

import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.Expression;
import java.util.ArrayList;
import java.util.stream.Collectors;

public class SubatoPrincipalSpecifications {

    public static CriteriaBuilder.In<Long> participantIn(SubatoPrincipal principal, CriteriaBuilder cb, Expression<Long> courseIdExpr) {
        var participatingCourseIds = principal != null ? principal.getParticipatingUnits().stream()
                .map(cu -> cu.getCourseId())
                .distinct()
                .collect(Collectors.toList()) : new ArrayList<Long>();

        var participantInBuilder = cb.in(courseIdExpr);
        participatingCourseIds.forEach(courseId -> participantInBuilder.value(courseId));

        return participantInBuilder;
    }

    public static CriteriaBuilder.In<Long> courseStaffIn(SubatoPrincipal principal, CriteriaBuilder cb, Expression<Long> courseIdExpr) {
        var staffCourseIds = principal != null ? principal.getStaffMemberships().stream()
                .map(m -> m.getCourseId())
                .distinct()
                .collect(Collectors.toList()) : new ArrayList<Long>();

        var courseStaffInBuilder = cb.in(courseIdExpr);
        staffCourseIds.forEach(courseId -> courseStaffInBuilder.value(courseId));

        return courseStaffInBuilder;
    }
}
