package de.hsrm.sls.subato.auth.policy.attendance;

import de.hsrm.sls.subato.auth.policy.CheckPolicyRequest;
import de.hsrm.sls.subato.auth.policy.PolicyRule;
import de.hsrm.sls.subato.courses.course.Course;
import de.hsrm.sls.subato.courses.membership.StaffGroup;
import de.hsrm.sls.subato.user.User;
import org.apache.commons.lang3.tuple.Pair;
import org.springframework.stereotype.Component;

/**
 * The current user is allowed to view attendances within a course
 */
@Component
public class AttendanceViewPolicy implements PolicyRule {

    @Override
    public boolean check(CheckPolicyRequest request) {
        if (!request.matches(AttendancePermission.ATTENDANCE_VIEW) || request.unauthenticated()) {
            return false;
        }

        var req = (Pair<Course, User>) request.getObj();
        var course = req.getLeft();
        var user = req.getRight();

        return user.getId() == request.getSubject().getId() || request.getSubject().hasAnyGroup(course.getId());
    }
}
