package de.hsrm.sls.subato.auth.policy.exercises;

import de.hsrm.sls.subato.auth.policy.SubatoPrincipalSpecifications;
import de.hsrm.sls.subato.auth.principal.CourseStaffMembership;
import de.hsrm.sls.subato.auth.principal.SubatoPrincipal;
import de.hsrm.sls.subato.auth.policy.CheckPolicyRequest;
import de.hsrm.sls.subato.auth.policy.PolicyRule;
import de.hsrm.sls.subato.courses.course.Course_;
import de.hsrm.sls.subato.courses.membership.CourseStaffMembership_;
import de.hsrm.sls.subato.exercises.exercise.Exercise_;
import de.hsrm.sls.subato.exercises.submission.Submission;
import de.hsrm.sls.subato.exercises.submission.Submission_;
import de.hsrm.sls.subato.user.User_;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Component;

import javax.persistence.criteria.JoinType;

/**
 * The current user is allowed to view a submission
 */
@Component
public class SubmissionViewPolicy implements PolicyRule {
    @Override
    public boolean check(CheckPolicyRequest request) {
        if(!request.matches(ExercisesAction.SUBMISSION_VIEW) || request.unauthenticated()) {
            return false;
        }

        var submission = (Submission) request.getObj();
        return submission.getSubmittedBy().getId() == request.getSubject().getId() || request.getSubject().hasAnyGroup(submission.getExercise().getCourse().getId());
    }

    @Override
    public Specification checkAsSpec(CheckPolicyRequest request) {
        if(!request.matches(ExercisesAction.SUBMISSION_VIEW)) {
            return Specification.where(null);
        }

        return accessesOwnSubmission(request.getSubject()).or(hasAnyGroup(request.getSubject()));
    }

    public static Specification<Submission> accessesOwnSubmission(SubatoPrincipal principal) {
        return (ex, cq, cb) -> cb.equal(ex.get(Submission_.submittedBy).get(User_.id), principal != null ? principal.getId() : -1);
    }

    public static Specification<Submission> hasAnyGroup(SubatoPrincipal principal) {
        return (ex, cq, cb) -> {
            var exerciseCourseJoin = ex
                    .join(Submission_.exercise)
                    .join(Exercise_.course);

            var courseStaffInBuilder = SubatoPrincipalSpecifications.courseStaffIn(principal, cb, exerciseCourseJoin.get(Course_.id));
            return courseStaffInBuilder;
        };
    }
}
