package de.hsrm.sls.subato.courses.course;

/**
 * Customization option for supplementing courses immediately after creation. May be used to import course units
 * or other data from another system.
 *
 * The identifier of the supplement implementation has to be
 * configured as the active supplement in application.properties
 * in order to be used. See {@link CourseSupplementFactory}
 *
 * To disable the course supplementation, use null (or "") as identifier in application.properties
 */
public interface CourseSupplement {
    /**
     * Supplements the given course and returns the supplemented course.
     *
     * Returning the modified course is required because
     * it is a hibernate entity that may require reassigment (through repository.save())
     */
    Course supplement(Course course);

    /**
     * Returns the unique identifier of this supplement (choose one by yourself)
     */
    String getIdentifier();
}
