package de.hsrm.sls.subato.courses.file;

import de.hsrm.sls.subato.auth.policy.CheckPolicyRequest;
import de.hsrm.sls.subato.auth.policy.PolicyEvaluator;
import de.hsrm.sls.subato.auth.policy.courses.CoursesAction;
import de.hsrm.sls.subato.courses.course.CourseNotFoundException;
import de.hsrm.sls.subato.courses.course.CourseService;
import de.hsrm.sls.subato.shared.FileResponseBuilder;
import de.hsrm.sls.subato.shared.fs.FileContent;
import de.hsrm.sls.subato.shared.helper.ResourceHelper;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.Resource;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.server.ResponseStatusException;

import java.io.IOException;

import static org.springframework.http.HttpStatus.BAD_REQUEST;
import static org.springframework.http.HttpStatus.NOT_FOUND;

@RestController
@Tag(name = "Kursverwaltung")
public class CourseFileController {
    @Autowired
    private CourseService courseService;

    @Autowired
    private CourseFileService courseFileService;

    @Autowired
    private CourseFileMapper mapper;

    @Autowired
    private PolicyEvaluator policy;

    @Autowired
    private FileResponseBuilder fileResponseBuilder;

    @GetMapping("/course_file/{id}/download")
    ResponseEntity<Resource> download(@PathVariable Long id) throws IOException {
        var courseFile = courseFileService.findById(id);
        policy.checkAndRaise(CheckPolicyRequest.fromPrincipal(courseFile.getCourse(), CoursesAction.COURSE_VIEW));

        var file = courseFileService.getFile(courseFile);
        var resource = ResourceHelper.load(file);
        return fileResponseBuilder.build(resource, true);
    }

    @PostMapping("/course_file")
    CourseFileDto create(@RequestPart("file") MultipartFile file, @RequestPart("metadata") CreateCourseFileDto dto) {
        try {
            var course = courseService.findById(dto.getCourseId());
            policy.checkAndRaise(CheckPolicyRequest.fromPrincipal(course, CoursesAction.COURSE_UPDATE));

            var courseFile = courseFileService.create(FileContent.fromMultipartFile(file), course, dto);
            return mapper.map(courseFile);
        } catch(CourseNotFoundException ex) {
            throw new ResponseStatusException(BAD_REQUEST, CourseNotFoundException.REASON);
        }
    }

    @DeleteMapping("/course_file/{id}")
    void delete(@PathVariable Long id) {
        try {
            var courseFile = courseFileService.findById(id);
            policy.checkAndRaise(CheckPolicyRequest.fromPrincipal(courseFile.getCourse(), CoursesAction.COURSE_UPDATE));

            courseFileService.delete(courseFile);
        } catch(CourseFileNotFoundException ex) {
            throw new ResponseStatusException(NOT_FOUND, CourseFileNotFoundException.REASON);
        }
    }
}
