package de.hsrm.sls.subato.courses.membership;

import de.hsrm.sls.subato.auth.principal.PrincipalService;
import de.hsrm.sls.subato.courses.course.Course;
import de.hsrm.sls.subato.courses.course.CourseRepository;
import de.hsrm.sls.subato.user.User;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.stereotype.Service;

@Service
public class CourseStaffMembershipService {
    @Autowired
    PrincipalService principalService;

    @Autowired
    CourseStaffMembershipRepository staffMembershipRepository;

    @Autowired
    private ApplicationEventPublisher applicationEventPublisher;

    public boolean hasMembership(Course course, User user, StaffGroup group) {
        return course.getMemberships().stream()
                .filter(m -> m.getUser().getId() == user.getId() && m.getGroup() == group)
                .count() != 0;
    }

    /**
     * @throws CourseStaffMembershipAlreadyExistsException
     */
    public CourseStaffMembership addMembership(Course course, User user, StaffGroup group) {
        if(hasMembership(course, user, group)) {
            throw new CourseStaffMembershipAlreadyExistsException();
        }

        var membership = new CourseStaffMembership(user, course, group);

        membership = staffMembershipRepository.save(membership);
        principalService.evictPrincipal(user);
        course.getMemberships().add(membership);

        applicationEventPublisher.publishEvent(new CourseStaffMembershipAddedEvent(this, membership));
        return membership;
    }

    /**
     * @throws CourseStaffMembershipNotFoundException
     */
    public void removeMembership(Course course, User user, StaffGroup group) {
        var membership = course.getMemberships().stream()
                .filter(m -> m.getUser().getId() == user.getId() && m.getGroup() == group)
                .findFirst().orElse(null);

        if(membership == null) {
            throw new CourseStaffMembershipNotFoundException();
        }

        course.getMemberships().remove(membership);
        membership = staffMembershipRepository.save(membership);
        principalService.evictPrincipal(membership.getUser());

        applicationEventPublisher.publishEvent(new CourseStaffMembershipRemovedEvent(this, membership));
    }
}
