package de.hsrm.sls.subato.exercises.submission.progress;

import de.hsrm.sls.subato.exercises.exercise.AssessmentMode;
import de.hsrm.sls.subato.exercises.submission.Submission;

import java.util.List;

public class SubmissionsProgress {
    private long passedTotal;
    private long notPassedTotal;
    private long undecidedTotal;
    private long total;

    private long fullyRatedTotal;

    public static SubmissionsProgress fromSubmissions(List<Submission> submissions) {
        var submissionsWithBinaryMode = submissions.stream()
                .filter(s -> s.getExercise().getAssessmentMode() == AssessmentMode.Binary)
                .toList();

        var progress = new SubmissionsProgress();
        progress.setPassedTotal(submissionsWithBinaryMode.stream().filter(s -> s.getPassed() != null && s.getPassed()).count());
        progress.setNotPassedTotal(submissionsWithBinaryMode.stream().filter(s -> s.getPassed() != null && !s.getPassed()).count());
        progress.setUndecidedTotal(submissionsWithBinaryMode.stream().filter(s -> s.getPassed() == null).count());
        progress.setTotal(submissions.size());

        var submissionsWithPointsMode = submissions.stream()
                .filter(s -> s.getExercise().getAssessmentMode() == AssessmentMode.Points)
                .toList();
        progress.setFullyRatedTotal(submissionsWithPointsMode.stream().filter(s -> s.getPointRatingCount() == s.getExercise().getTaskInstancesCount()).count());

        return progress;
    }

    public long getPassedTotal() {
        return passedTotal;
    }

    public void setPassedTotal(long passedTotal) {
        this.passedTotal = passedTotal;
    }

    public long getNotPassedTotal() {
        return notPassedTotal;
    }

    public void setNotPassedTotal(long notPassedTotal) {
        this.notPassedTotal = notPassedTotal;
    }

    public long getUndecidedTotal() {
        return undecidedTotal;
    }

    public void setUndecidedTotal(long undecidedTotal) {
        this.undecidedTotal = undecidedTotal;
    }

    public long getTotal() {
        return total;
    }

    public void setTotal(long total) {
        this.total = total;
    }

    public long getFullyRatedTotal() {
        return fullyRatedTotal;
    }

    public void setFullyRatedTotal(long fullyRatedTotal) {
        this.fullyRatedTotal = fullyRatedTotal;
    }
}
