package de.hsrm.sls.subato.shared.fs;

import net.lingala.zip4j.ZipFile;
import net.lingala.zip4j.exception.ZipException;
import org.springframework.web.multipart.MultipartFile;

import java.io.*;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;

public class FileContent {
    private InputStream stream;
    private String fileName;

    public FileContent(InputStream stream, String fileName) {
        this.stream = stream;
        this.fileName = fileName;
    }

    public String getFileName() {
        return fileName;
    }

    public InputStream getStream() {
        return stream;
    }

    public static FileContent fromMultipartFile(MultipartFile file) {
        try {
            return new FileContent(file.getInputStream(), file.getOriginalFilename());
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    public static FileContent fromFile(File file) {
        try {
            return new FileContent(new FileInputStream(file), file.getName());
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    public static FileContent fromString(String name, String content) {
        var stream = new ByteArrayInputStream(content.getBytes(StandardCharsets.UTF_8));
        return new FileContent(stream, name);
    }

    public void saveTo(File dir) {
        saveTo(dir, false);
    }

    public void saveTo(File dir, boolean extractZip) {
        try {
            var dirPath = dir.toPath();
            var path = dirPath.resolve(fileName);
            Files.createDirectories(dirPath);
            Files.copy(stream, path);

            var zipFile = new ZipFile(path.toString());

            if(extractZip && zipFile.isValidZipFile()) {
                zipFile.extractAll(dirPath.toString());
                Files.delete(path);
            }
        } catch (ZipException e) {
            throw new RuntimeException(e);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }
}
