package de.hsrm.sls.subato.solutions.solution;

import de.hsrm.sls.subato.exercises.exercise.Exercise_;
import de.hsrm.sls.subato.exercises.submission.Submission_;
import de.hsrm.sls.subato.tasks.task.Task_;
import de.hsrm.sls.subato.user.User_;
import org.springframework.data.jpa.domain.Specification;

/**
 * Used for dynamic filtering in {@link SolutionService#findAll(SolutionFilter)}
 */
public class SolutionSpecifications {
    public static Specification<Solution> submittedByEquals(long submittedById) {
        return (ex, cq, cb) -> cb.equal(ex.get(Solution_.submittedBy).get(User_.id), submittedById);
    }

    public static Specification<Solution> exerciseEquals(long exerciseId) {
        return (ex, cq, cb) -> cb.equal(ex.get(Solution_.exercise).get(Exercise_.id), exerciseId);
    }

    public static Specification<Solution> taskEquals(long taskId) {
        return (ex, cq, cb) -> cb.equal(ex.get(Solution_.task).get(Task_.id), taskId);
    }

    public static Specification<Solution> submissionEquals(long submissionId) {
        return (ex, cq, cb) -> cb.equal(ex.get(Solution_.submission).get(Submission_.id), submissionId);
    }

    public static Specification<Solution> isValid(boolean valid) {
        if(valid) {
            return (ex, cq, cb) -> cb.and(cb.equal(ex.get(Solution_.delayed), false), cb.equal(ex.get(Solution_.attemptsExceeded), false));
        } else {
            return (ex, cq, cb) -> cb.or(cb.equal(ex.get(Solution_.delayed), true), cb.equal(ex.get(Solution_.attemptsExceeded), true));
        }
    }
}
