package de.hsrm.sls.subato.aor.core;

import de.hsrm.sls.subato.Tags;
import de.hsrm.sls.subato.modules.progmodule.ProgramModuleRepository;
import de.hsrm.sls.subato.modules.program.DegreeProgramRepository;
import org.junit.Assert;
import org.junit.jupiter.api.Tag;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.jdbc.AutoConfigureTestDatabase;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.annotation.DirtiesContext;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.ResourceUtils;

import java.nio.file.Files;

/**
 * ad-hoc integration tests, should be refactored someday to test the individual import steps isolated
 */
@SpringBootTest
@DirtiesContext(classMode = DirtiesContext.ClassMode.BEFORE_EACH_TEST_METHOD)
@AutoConfigureTestDatabase(replace = AutoConfigureTestDatabase.Replace.ANY)
@Tag(Tags.INTEGRATION)
@Transactional
public class CoreDataImportManagerTests {
    @Autowired
    private CoreDataImportManager coreDataImportManager;

    @Autowired
    private ProgramModuleRepository programModuleRepository;

    @Autowired
    private DegreeProgramRepository programRepository;

    @Test
    public void shouldImport() throws Exception {
        // remember how many programs/modules were already in the fixtures
        var existingPrograms = programRepository.count();
        var existingModules = programModuleRepository.count();

        var sampleImportFile = ResourceUtils.getFile("classpath:testfiles/aor/core-data.json");
        coreDataImportManager.importFromJson(Files.readString(sampleImportFile.toPath()));

        // check the delta: how many have been added
        Assert.assertEquals(2, programRepository.count() - existingPrograms);
        Assert.assertEquals(4, programModuleRepository.count() - existingModules);

        // thesis should be ignored
        Assert.assertNull(programModuleRepository.findByAorId(1247808L));

        // should be deactivated because they are not in core-data.json
        Assert.assertFalse(programRepository.findByPosCode("IAB").isActive());
        Assert.assertFalse(programRepository.findByPosCode("IBB").isActive());
    }
}
