package de.hsrm.sls.subato.courses.course;

import de.hsrm.sls.subato.Tags;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Tag;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.jdbc.AutoConfigureTestDatabase;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithUserDetails;
import org.springframework.test.annotation.DirtiesContext;
import org.springframework.test.web.servlet.MockMvc;

import static de.hsrm.sls.subato.utils.JsonHelper.asJsonString;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.put;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

/**
 * Integration Test für {@link CourseController#update(Long, UpdateCourseDto)}
 */
@SpringBootTest
@AutoConfigureMockMvc
@DirtiesContext(classMode = DirtiesContext.ClassMode.BEFORE_EACH_TEST_METHOD)
@AutoConfigureTestDatabase(replace = AutoConfigureTestDatabase.Replace.ANY)
@Tag(Tags.INTEGRATION)
public class UpdateCourseTest {
    @Autowired
    private MockMvc mockMvc;

    private UpdateCourseDto update;

    @BeforeEach
    public void setup() {
        update = new UpdateCourseDto();
        update.setName("Name");
        update.setDescription("Beschreibung");
        update.setShortName("Kurzname");
        update.setVisibility(CourseVisibility.PRIVATE);
        update.setSubmitPolicy(SubmitPolicy.PARTICIPANTS);
    }

    @Test
    @WithUserDetails("US_C1A")
    public void shouldAllowCourseAdmins() throws Exception {
        this.mockMvc.perform(put("/course/1")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(asJsonString(update)))
                .andDo(print()).andExpect(status().isOk());
    }

    @Test
    public void shouldDenyGuests() throws Exception {
        this.mockMvc.perform(put("/course/1")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(asJsonString(update)))
                .andDo(print()).andExpect(status().isUnauthorized());
    }
}
